@@ -115,6 +115,8 @@ static void gnc_plugin_page_budget_cmd_view_options(
     GtkAction *action, GncPluginPageBudget *page);
 static void gnc_plugin_page_budget_cmd_estimate_budget(
     GtkAction *action, GncPluginPageBudget *page);
+static void gnc_plugin_page_budget_cmd_allperiods_budget(
+    GtkAction *action, GncPluginPageBudget *page);
 
 static GtkActionEntry gnc_plugin_page_budget_actions [] =
 {
@@ -151,6 +153,12 @@ static GtkActionEntry gnc_plugin_page_budget_actions [] =
         N_("Estimate a budget value for the selected accounts from past transactions"),
         G_CALLBACK (gnc_plugin_page_budget_cmd_estimate_budget)
     },
+    {
+        "AllPeriodsBudgetAction", "system-run", N_("All Periods"),
+        NULL,
+        N_("Edit budget for all periods for the selected accounts"),
+        G_CALLBACK (gnc_plugin_page_budget_cmd_allperiods_budget)
+    },
 
     /* View menu */
     {
@@ -179,9 +187,18 @@ static action_toolbar_labels toolbar_labels[] =
     { "DeleteBudgetAction",         N_("Delete") },
     { "OptionsBudgetAction",        N_("Options") },
     { "EstimateBudgetAction",       N_("Estimate") },
+    { "AllPeriodsBudgetAction",     N_("All Periods") },
     { NULL, NULL },
 };
 
+typedef enum allperiods_action 
+{
+    REPLACE,
+    ADD,
+    MULTIPLY,
+    UNSET
+} allperiods_action; 
+
 typedef struct GncPluginPageBudgetPrivate
 {
     GtkActionGroup *action_group;
@@ -204,43 +221,21 @@ typedef struct GncPluginPageBudgetPrivate
     /* For the estimation dialog */
     Recurrence r;
     gint sigFigs;
+    gboolean useAvg;
+
+    /* For the allPeriods value dialog */
+    gnc_numeric allValue;
+    allperiods_action action;
+
 } GncPluginPageBudgetPrivate;
 
+G_DEFINE_TYPE_WITH_PRIVATE(GncPluginPageBudget, gnc_plugin_page_budget, GNC_TYPE_PLUGIN_PAGE)
+
 #define GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNC_TYPE_PLUGIN_PAGE_BUDGET, GncPluginPageBudgetPrivate))
 
 static GObjectClass *parent_class = NULL;
 
-
-GType
-gnc_plugin_page_budget_get_type (void)
-{
-    static GType gnc_plugin_page_budget_type = 0;
-
-    if (gnc_plugin_page_budget_type == 0)
-    {
-        static const GTypeInfo our_info =
-        {
-            sizeof (GncPluginPageBudgetClass),
-            NULL,
-            NULL,
-            (GClassInitFunc) gnc_plugin_page_budget_class_init,
-            NULL,
-            NULL,
-            sizeof (GncPluginPageBudget),
-            0,
-            (GInstanceInitFunc) gnc_plugin_page_budget_init
-        };
-
-        gnc_plugin_page_budget_type =
-            g_type_register_static (GNC_TYPE_PLUGIN_PAGE,
-                                    "GncPluginPageBudget", &our_info, 0);
-    }
-
-    return gnc_plugin_page_budget_type;
-}
-
-
 GncPluginPage *
 gnc_plugin_page_budget_new (GncBudget *budget)
 {
@@ -295,8 +290,6 @@ gnc_plugin_page_budget_class_init (GncPluginPageBudgetClass *klass)
     gnc_plugin_class->destroy_widget  = gnc_plugin_page_budget_destroy_widget;
     gnc_plugin_class->save_page       = gnc_plugin_page_budget_save_page;
     gnc_plugin_class->recreate_page   = gnc_plugin_page_budget_recreate_page;
-
-    g_type_class_add_private(klass, sizeof(GncPluginPageBudgetPrivate));
 }
 
 
@@ -334,10 +327,12 @@ gnc_plugin_page_budget_init (GncPluginPageBudget *plugin_page)
     /* Visible types */
     priv->fd.visible_types = -1; /* Start with all types */
     priv->fd.show_hidden = FALSE;
+    priv->fd.show_unused = TRUE;
     priv->fd.show_zero_total = TRUE;
     priv->fd.filter_override = g_hash_table_new (g_direct_hash, g_direct_equal);
 
     priv->sigFigs = 1;
+    priv->useAvg = FALSE;
     recurrenceSet(&priv->r, 1, PERIOD_MONTH, NULL, WEEKEND_ADJ_NONE);
 
     LEAVE("page %p, priv %p, action group %p",
@@ -479,6 +474,9 @@ gnc_plugin_page_budget_destroy_widget (GncPluginPage *plugin_page)
 
     if (priv->budget_view)
     {
+        // save the account filter state information to budget section
+        gnc_budget_view_save_account_filter (priv->budget_view);
+
         if (priv->delete_budget)
         {
             gnc_budget_view_delete_budget (priv->budget_view);
@@ -537,7 +535,7 @@ gnc_plugin_page_budget_save_page (GncPluginPage *plugin_page,
     guid_to_string_buff(gnc_budget_get_guid(priv->budget), guid_str);
     g_key_file_set_string(key_file, group_name, BUDGET_GUID, guid_str);
 
-    //FIXME
+    // Save the Budget page information to state file
     gnc_budget_view_save(priv->budget_view, key_file, group_name);
 
     LEAVE(" ");
@@ -885,19 +883,40 @@ estimate_budget_helper(GtkTreeModel *model, GtkTreePath *path,
 
     num_periods = gnc_budget_get_num_periods(priv->budget);
 
-    for (i = 0; i < num_periods; i++)
+    if (priv->useAvg && num_periods)
     {
-        num = recurrenceGetAccountPeriodValue(&priv->r, acct, i);
-        if (!gnc_numeric_check(num))
+        num = xaccAccountGetBalanceChangeForPeriod(acct, 
+                recurrenceGetPeriodTime(&priv->r, 0, FALSE),
+                recurrenceGetPeriodTime(&priv->r, num_periods - 1, TRUE), TRUE);
+        num = gnc_numeric_div(num, 
+                              gnc_numeric_create(num_periods, 1), 
+                              GNC_DENOM_AUTO,
+                              GNC_HOW_DENOM_SIGFIGS(priv->sigFigs) | 
+                              GNC_HOW_RND_ROUND_HALF_UP);
+
+        if (gnc_reverse_balance(acct))
+            num = gnc_numeric_neg(num);
+
+        for (i = 0; i < num_periods; i++)
         {
-            if (gnc_reverse_balance (acct))
-                num = gnc_numeric_neg (num);
+            gnc_budget_set_account_period_value(priv->budget, acct, i, num);
+        }
+    }
+    else
+    {
+        for (i = 0; i < num_periods; i++)
+        {
+            num = recurrenceGetAccountPeriodValue(&priv->r, acct, i);
+            if (!gnc_numeric_check(num))
+            {
+                if (gnc_reverse_balance(acct))
+                    num = gnc_numeric_neg(num);
 
-
-            num = gnc_numeric_convert(num, GNC_DENOM_AUTO,
-                                      GNC_HOW_DENOM_SIGFIGS(priv->sigFigs) | GNC_HOW_RND_ROUND_HALF_UP);
-            gnc_budget_set_account_period_value(
-                priv->budget, acct, i, num);
+                num = gnc_numeric_convert(num, GNC_DENOM_AUTO,
+                                          GNC_HOW_DENOM_SIGFIGS(priv->sigFigs) |
+                                          GNC_HOW_RND_ROUND_HALF_UP);
+                gnc_budget_set_account_period_value(priv->budget, acct, i, num);
+            }
         }
     }
 }
@@ -912,7 +931,7 @@ gnc_plugin_page_budget_cmd_estimate_budget(GtkAction *action,
 {
     GncPluginPageBudgetPrivate *priv;
     GtkTreeSelection *sel;
-    GtkWidget *dialog, *gde, *dtr, *hb;
+    GtkWidget *dialog, *gde, *dtr, *hb, *avg;
     gint result;
     GDate date;
     const Recurrence *r;
@@ -956,6 +975,9 @@ gnc_plugin_page_budget_cmd_estimate_budget(GtkAction *action,
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(dtr),
                               (gdouble)priv->sigFigs);
 
+    avg = GTK_WIDGET(gtk_builder_get_object(builder, "UseAverage"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(avg), priv->useAvg);
+
     gtk_widget_show_all (dialog);
     result = gtk_dialog_run(GTK_DIALOG(dialog));
     switch (result)
@@ -970,6 +992,8 @@ gnc_plugin_page_budget_cmd_estimate_budget(GtkAction *action,
         priv->sigFigs =
             gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dtr));
 
+        priv->useAvg = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(avg));
+        
         gtk_tree_selection_selected_foreach(sel, estimate_budget_helper, page);
         break;
     default:
@@ -979,6 +1003,133 @@ gnc_plugin_page_budget_cmd_estimate_budget(GtkAction *action,
     g_object_unref(G_OBJECT(builder));
 }
 
+static void
+allperiods_budget_helper(GtkTreeModel *model, GtkTreePath *path,
+                         GtkTreeIter *iter, gpointer data)
+{
+    Account *acct;
+    guint num_periods, i;
+    gnc_numeric num;
+    GncPluginPageBudgetPrivate *priv;
+    GncPluginPageBudget *page = data;
+
+    g_return_if_fail(GNC_IS_PLUGIN_PAGE_BUDGET(page));
+    priv = GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(page);
+    acct = gnc_budget_view_get_account_from_path(priv->budget_view, path);
+    num_periods = gnc_budget_get_num_periods(priv->budget);
+    num = priv->allValue;
+
+    for (i = 0; i < num_periods; i++)
+    {
+        switch (priv->action)
+        {
+        case ADD:
+            num = gnc_budget_get_account_period_value(priv->budget, acct, i);
+            num = gnc_numeric_add(num, priv->allValue, GNC_DENOM_AUTO,
+                                  GNC_HOW_DENOM_SIGFIGS(priv->sigFigs) |
+                                  GNC_HOW_RND_ROUND_HALF_UP);
+            gnc_budget_set_account_period_value(priv->budget, acct, i, num);
+            break;
+        case MULTIPLY:
+            num = gnc_budget_get_account_period_value(priv->budget, acct, i);
+            num = gnc_numeric_mul(num, priv->allValue, GNC_DENOM_AUTO,
+                                  GNC_HOW_DENOM_SIGFIGS(priv->sigFigs) |
+                                  GNC_HOW_RND_ROUND_HALF_UP);
+            gnc_budget_set_account_period_value(priv->budget, acct, i, num);
+            break;
+        case UNSET:
+            gnc_budget_unset_account_period_value(priv->budget, acct, i);
+            break;
+        default:
+            gnc_budget_set_account_period_value(priv->budget, acct, i,
+                                                priv->allValue);
+            break;
+        }
+    }
+}
+
+/*******************************/
+/*  All Periods Value Dialog   */
+/*******************************/
+static void
+gnc_plugin_page_budget_cmd_allperiods_budget(GtkAction *action,
+                                             GncPluginPageBudget *page)
+{
+    GncPluginPageBudgetPrivate *priv;
+    GtkTreeSelection *sel;
+    GtkWidget *dialog, *gde, *val, *dtr, *add, *mult;
+    gint result;
+    GtkBuilder *builder;
+    const gchar *txt;
+
+    g_return_if_fail(GNC_IS_PLUGIN_PAGE_BUDGET(page));
+    priv = GNC_PLUGIN_PAGE_BUDGET_GET_PRIVATE(page);
+    sel = gnc_budget_view_get_selection(priv->budget_view);
+
+    if (gtk_tree_selection_count_selected_rows(sel) <= 0)
+    {
+        dialog = gtk_message_dialog_new(
+                    GTK_WINDOW(gnc_plugin_page_get_window(GNC_PLUGIN_PAGE(page))),
+                    GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                    GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "%s",
+                    _("You must select at least one account to edit."));
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
+        return;
+    }
+
+    builder = gtk_builder_new();
+    gnc_builder_add_from_file(builder, "gnc-plugin-page-budget.glade",
+                              "DigitsToRound_Adj");
+    gnc_builder_add_from_file(builder, "gnc-plugin-page-budget.glade",
+                              "budget_allperiods_dialog");
+
+    dialog = GTK_WIDGET(
+        gtk_builder_get_object(builder, "budget_allperiods_dialog"));
+
+    gtk_window_set_transient_for(
+        GTK_WINDOW(dialog),
+        GTK_WINDOW(gnc_plugin_page_get_window(GNC_PLUGIN_PAGE(page))));
+
+    val = GTK_WIDGET(gtk_builder_get_object(builder, "Value"));
+    gtk_entry_set_text(GTK_ENTRY(val), "");
+
+    dtr = GTK_WIDGET(gtk_builder_get_object(builder, "DigitsToRound1"));
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(dtr), (gdouble)priv->sigFigs);
+
+    add  = GTK_WIDGET(gtk_builder_get_object(builder, "RB_Add"));
+    mult = GTK_WIDGET(gtk_builder_get_object(builder, "RB_Multiply"));
+
+    gtk_widget_show_all(dialog);
+    result = gtk_dialog_run(GTK_DIALOG(dialog));
+    switch (result)
+    {
+    case GTK_RESPONSE_OK:
+
+        priv->sigFigs = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dtr));
+        priv->action = REPLACE;
+        txt = gtk_entry_get_text(GTK_ENTRY(val));
+
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add)))
+            priv->action = ADD;
+        else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mult)))
+            priv->action = MULTIPLY;
+
+        if (priv->action == REPLACE &&
+            !gtk_entry_get_text_length(GTK_ENTRY(val)))
+            priv->action = UNSET;
+
+        if (xaccParseAmount(txt, TRUE, &priv->allValue, NULL) ||
+            priv->action == UNSET)
+            gtk_tree_selection_selected_foreach(sel, allperiods_budget_helper,
+                                                page);
+        break;
+    default:
+        break;
+    }
+    gtk_widget_destroy(dialog);
+    g_object_unref(G_OBJECT(builder));
+}
 
 static void
 gnc_plugin_page_budget_cmd_view_filter_by (GtkAction *action,
